
--------------------------------------------------------------------------------
-- Made by (Jip) Willem Wijnia
-- See also: https://gitlab.com/w.b.wijnia/rainmakers
-- 
-- Please do not remove this message when modifying the map.
--------------------------------------------------------------------------------

------------------ ------------------ ------------------ ------------------ ------------------                              
-- IMPORTS --
------------------ ------------------ ------------------ ------------------ ------------------ 

local ScenarioFramework = import('/lua/ScenarioFramework.lua')
local Objectives = import('/lua/ScenarioFramework.lua').Objectives

------------------ ------------------ ------------------ ------------------ ------------------                              
-- TWEAKABLE VALUES --
------------------ ------------------ ------------------ ------------------ ------------------ 

local titleTimerObjective = 'Time to hold out';
local descriptionTimerObjective = 'The time left for the installations to assist the off-site siege.';

local titleProtectObjective = "Protect the artillery installations.";
local descriptionProtectObjective = "These installations are vital to an off-site siege. Protect them at all costs.";

------------------ ------------------ ------------------ ------------------ ------------------                              
-- NON-TWEAKABLE VALUES --
------------------ ------------------ ------------------ ------------------ ------------------ 

local X05_DB01_030 = { {text = '<LOC X05_DB01_030_010>[{i HQ}]: The operation has ended in failure. All is lost.', vid = 'X05_HQ_DB01_04956.sfd', bank = 'Briefings', cue = 'X05_HQ_DB01_04956', faction = 'NONE'}, }

------------------ ------------------ ------------------ ------------------ ------------------                              
-- FUNCTIONALITY --
------------------ ------------------ ------------------ ------------------ ------------------ 

Event = { };
Event.__index = Event;

-- initialises the event.
function Event:Setup()

    -- linking
    event = { };
    setmetatable(event, Event);

    return event;
end

function Event:Launch(eventInfo)

    -- the timer actually counts down in this version!
    self.TimerObjective = Objectives.Timer(
        'primary',
        'incomplete',
        titleTimerObjective,
        descriptionTimerObjective,
        {
            Timer = ScenarioInfo.Config.GeneralSettings.ObjectiveTime + ScenarioInfo.Config.GeneralSettings.StartupTime,                  -- time in seconds
            ExpireResult = 'complete',
        }
    );

    self.TimerObjective:AddResultCallback(
        function(result)
            if result then

                -- prevent them from attacking.
                for k, objective in ScenarioInfo.Globals.ArtilleryObjectives do
                    objective:SetFireState(3);
                end

                -- complete the protection objective.
                self.ProtectObjective:ManualResult(true);
            else
                -- do nothing.
            end
        end
    );

    -- make 'dem visible!
    for l, unit in ScenarioInfo.Globals.ArtilleryObjectives do
        for k, player in eventInfo.StrPlayers do
            ScenarioFramework.CreateVisibleAreaAtUnit(30, unit, ScenarioInfo.Config.GeneralSettings.ObjectiveTime, GetArmyBrain(player));
        end
    end

    -- prepare the protecting objective, add in some baking powder.
    self.ProtectObjective = Objectives.Protect(
        'primary',                      
        'incomplete',                   
        titleProtectObjective,  
        descriptionProtectObjective,  
        {                               
            Units = ScenarioInfo.Globals.ArtilleryObjectives,
        }
    );

    -- determine the consequences of our actions.
    self.ProtectObjective:AddResultCallback(
    function(result)
        if result then

            -- succes!

            -- we have no hollywood scenes for this. HQ doesn't appear to be appreciative in general.
            --ScenarioFramework.Dialogue(???, nil, false);

            -- start the recall event! Time to get cooking.
            ScenarioFramework.CreateTimerTrigger(
                function() 
                    eventInfo.OtherEvents.Recall:Launch(eventInfo);
                end,
                3,
                true
            );
            
        else
            -- failure!
            self.TimerObjective:ManualResult(false);

            -- do some Hollywood. About destruction. Boohoo.
            ScenarioFramework.Dialogue(X05_DB01_030, nil, false);

            -- end the game after a few seconds.
            ScenarioFramework.CreateTimerTrigger(
                function() 
                    for k, player in eventInfo.StrPlayers do
                        GetArmyBrain(player):OnDefeat();
                    end

                    EndGame();
                end,
                5.0,
                true
            );
        end
    end
    );
end