
--------------------------------------------------------------------------------
-- Made by (Jip) Willem Wijnia
-- See also: https://gitlab.com/w.b.wijnia/rainmakers
-- 
-- Please do not remove this message when modifying the map.
--------------------------------------------------------------------------------

------------------ ------------------ ------------------ ------------------ ------------------                              
-- IMPORTS --
------------------ ------------------ ------------------ ------------------ ------------------ 

local ScenarioFramework = import('/lua/ScenarioFramework.lua')
local Objectives = import('/lua/ScenarioFramework.lua').Objectives

local EventFunctionality = import(ScenarioInfo.path .. 'Events/EventFunctionality.lua');

------------------ ------------------ ------------------ ------------------ ------------------                              
-- TWEAKABLE VALUES --
------------------ ------------------ ------------------ ------------------ ------------------ 

local titleProtectObjective = 'Protect the commanders';
local descriptionProtectObjective = 'All commanders must survive - we leave no man behind.';

------------------ ------------------ ------------------ ------------------ ------------------                              
-- NON-TWEAKABLE VALUES --
------------------ ------------------ ------------------ ------------------ ------------------ 

-- sounds when one commander dies while all should live.
local X06_DB01_030 = { {text = '<LOC X06_DB01_030_010>[{i HQ}]: Commander? You there? Dammit.', vid = 'X06_HQ_DB01_04963.sfd', bank = 'Briefings', cue = 'X06_HQ_DB01_04963', faction = 'NONE'},}
local X06_M01_140 = { {text = '<LOC X06_M01_140_010>[{i HQ}]: Commander, you read me? Commander? Ah hell...', vid = 'X06_HQ_M01_04961.sfd', bank = 'X06_VO', cue = 'X06_HQ_M01_04961', faction = 'NONE'},}
local X05_DB01_040 = { {text = '<LOC X05_DB01_040_010>[{i HQ}]: Commander? Can you read me? Ah hell, I think the Commander is dea --', vid = 'X05_HQ_DB01_04957.sfd', bank = 'Briefings', cue = 'X05_HQ_DB01_04957', faction = 'NONE'},}

------------------ ------------------ ------------------ ------------------ ------------------                              
-- FUNCTIONALITY --
------------------ ------------------ ------------------ ------------------ ------------------ 

Event = { };
Event.__index = Event;

-- initialises the event.
function Event:Setup()

    -- linking
    event = { };
    setmetatable(event, Event);

    return event;
end

function Event:Launch(eventInfo)

    -- find all the commanders.
    local commanders = { };
    for k, player in eventInfo.StrPlayers do
        local brain = GetArmyBrain(player);
        local commander = brain:GetListOfUnits(categories.COMMAND, false);
        table.insert(commanders, commander[1]);
    end

    -- setup the protect objective.
    self.ObjectiveCommander = Objectives.Protect(
        'primary',
        'incomplete',
        titleProtectObjective,
        descriptionProtectObjective,
        {
            -- all the commanders must survive.
            Units = commanders,
            NumRequired = table.getn(commanders);
        }
    )

    -- determine the consequences.
    self.ObjectiveCommander:AddResultCallback(
        function(result)
            if result then
                -- succes!
            else
                -- play a random 'oh no, commander died' video.
                local dialogues = { X06_DB01_030, X06_M01_140, X05_DB01_040}
                EventFunctionality.PlayRandomDialogue(dialogues);
    
                -- we're done for :(.)
                ScenarioFramework.CreateTimerTrigger(
                    function()
                        EndGame();
                    end,
                    6,
                    true
                );
                
            end
        end
    );
end