
--------------------------------------------------------------------------------
-- Made by (Jip) Willem Wijnia
-- See also: https://gitlab.com/w.b.wijnia/rainmakers
-- 
-- Please do not remove this message when modifying the map.
--------------------------------------------------------------------------------

------------------ ------------------ ------------------ ------------------ ------------------                              
-- IMPORTS --
------------------ ------------------ ------------------ ------------------ ------------------ 

local ScenarioUtils = import('/lua/sim/ScenarioUtilities.lua')
local GeneralFunctionality = import(ScenarioInfo.path .. 'GeneralFunctionality.lua');

------------------ ------------------ ------------------ ------------------ ------------------                              
-- TWEAKABLE VALUES --
------------------ ------------------ ------------------ ------------------ ------------------ 

local debug = false;

------------------ ------------------ ------------------ ------------------ ------------------                              
-- END OF TWEAKABLE VALUES --
------------------ ------------------ ------------------ ------------------ ------------------ 

------------------ ------------------ ------------------ ------------------ ------------------                              
-- USEFULL FUNCTIONS --
------------------ ------------------ ------------------ ------------------ ------------------ 

-- analyses all spawning regions.
function AnalyseRegions()

    local areaTable = { };

    for k = 1, 50 do

        -- attempt to retrieve the area.
        local name = "SPAWN" .. k;
        local area = Scenario.Areas[name]

        -- if not, stop. We've found all our stream groups.
        if not area then
            break;
        end

        -- construct the rectangle
        local data = area.rectangle;
        local rect = Rect(data[1],data[2],data[3],data[4]);

        -- pre compute the center
        local cx = (rect.x1 + rect.x0) * 0.5;
        local cz = (rect.y1 + rect.y0) * 0.5;
        local cy = GetSurfaceHeight(cx, cz);
        rect.center = Vector(cx, cy,  cz);

        -- pre compute the line segment
        -- TODO

        -- add name for debugging
        rect.name = name;

        -- store it
        table.insert(areaTable, rect);
    end

    -- LOG(repr(areaTable));

    return areaTable;
end

-- retrieves a random point within a spawning region.
function RandomPointInRegion(area)
    return GeneralFunctionality.Vector(Random() * (area.x1 - area.x0) + area.x0, 0, Random() * (area.y1 - area.y0) + area.y0);
end

function ClosestRegionToPosition(regions, position)

    function ComputeDistance(a, b)
        local x = a[1] - b[1];
        local y = a[3] - b[3];

        return x * x + y * y;
    end

    -- find the closest region
    local ic = 1;
    local id = VDist2Sq(regions[1].center[1], regions[1].center[3], position[1], position[3]);

    for k, region in regions do 
        idn = VDist2Sq(region.center[1], region.center[3], position[1], position[3]);
        if idn < id then 
            ic = k;
            id = idn;
        end
    end

    -- return the region that is closest
    return regions[ic];
end